<?php

/**
 * Die Klasse {@link DataColumn}
 *
 * @package phpDataSet
 * @author Susanne Mckl <RezsiaDione@web.de>;
 * @since 23.05.2004
 */

/**
 * Diese Klasse {@link DataColumn} reprsentiert das Schema einer Spalte einer {@link DataTable}
 *
 * @package phpDataSet
 * @author Susanne Mckl <RezsiaDione@web.de>;
 * @since 23.05.2004
 */
class DataColumn
{
	//function DataColumn($strDataColumnName = null, $DataType = null, $strExpression = null, $strMappingType = null)
	/**
	* Der Konstruktor erstellt eine Instanz eines DataColumn-Objektes.
	* Eine DataColumn gehrt zu einer {@link DataTable}, die hier angegeben werden sollte.
	*
	* @access public
	* @param DataColumnName string optional
	* Wird ein DataColumnName angegeben wird er als Name der Column bernommen.
	* Wird keiner bergeben generiert der Konstrukor einen eindeutigen Namen fr die Column.
	*/
	function DataColumn(&$DataTable, $strDataColumnName = null )
	{
		static $objCounter = 0;

    	if(is_null($strDataColumnName))
		{
      		$this->strColumnName = "DataColumn" . $objCounter;
    	}
    	else
		{
     		$this->strColumnName = $strDataColumnName;
    	}
    	++$objCounter;
    	
        //Etwas wie "if($DataTable)" sollte man vermeiden: 
        //  http://www.php.net/manual/en/language.oop.constructor.php ab "Any object evaluation" ...
    	if(isset($DataTable))
    	{
    	    $this->DataTable =& $DataTable;
    	}
	}
	
	/**
	* Eindeutiger Name der DataColumn
	*
	* get/set-Methode dieses Attributs: {@link ColumnName() ColumnName}. Wird durch den Konstruktor
    	* erstmals und eindeutig festgelegt.
	* @access private
	* @var string
	*/
	var $strColumnName;

	/**
    	* Name der DataColumn (lesen, schreiben)
    	*
    	* get/set-Methode des Attributs {@link $strColumnName ColumnName}.
    	* @access public
    	* @param string Wert, auf den der Name der DataColumn gesetzt werden soll. Optional.
    	* @return string aktueller Name der DataColumn
    	*/
	function ColumnName($strDataColumnName = null)
	{
	    if(is_null($strDataColumnName))
	    {
	        return $this->strColumnName;
	    }
	    else
	    {
	        $this->strColumnName = $strDataColumnName;
	    }
	}
	
	/**
	* Spaltenberschrift
	*
	* get/set-Methode dieses Attributs: {@link Caption() Caption}.
	* @access private
    	* @var string
	*/
	var $strCaption = null;

	/**
	* berschrift der Spalte (lesen, schreiben)
	*
	* get/set-Methode des Attributs {@link $strCaption Caption}.
	* @access public
	* @param string optional Wert auf den die Variable Caption gesetzt werden soll.
	* @return string aktueller Wert von Caption, wenn dieser null ist aktueller Name der DataColumn
	*/
	function Caption($strCaption = null)
	{
	    if(is_null($strCaption))
	    {
	        if(is_null($this->strCaption))
	        {
	            return $this->strColumnName;
	        }
	        else
	        {
	            return $this->strCaption;
	        }
	    }
	    else
	    {
	        $this->strCaption = $strCaption;
	    }
	}
	
	/**
	* Default-Wert mit dem eine neu erzeugte DataRow gefllt wird
	*
	* get/set-Methode dieses Attributs: {@link DefaultValue() DefaultValue}.
	* @access private
	* @var string
	*/
	var $strDefaultValue = null;

	/**
	* Default-Wert der Spalte (lesen, schreiben)
	*
	* get/set-Methode des Attributs {@link $strDefaultValue DefaultValue}.
	* @access public
	* @param string optional Wert auf den die Variable DefaultValue gesetzt werden soll.
	* @return string aktueller Wert von DefaultValue
	*/
	function DefaultValue($strDefaultValue = null)
	{
	    if(is_null($strDefaultValue))
	    {
	        return $this->strDefaultValue;  
	    }
	    else
	    {
	        $this->strDefaultValue = $strDefaultValue;
	    }
	}
	
	/**
	* Maximale Lnge die der Wert haben darf
	*
	* get/set-Methode dieses Attributs: {@link MaxLength() MaxLength}.
	* @access private
	* @var int
	*/
	var $intMaxLength = null;

	/**
	* Maximale Lnge des Inhaltes der Spalte (lesen, schreiben)
	*
	* get/set-Methode des Attributs {@link $intMaxLength MaxLength}.
	* @access public
	* @param int optional Wert auf den die Variable MaxLength gesetzt werden soll.
	* @return int aktueller Wert von MaxLength, falls null kein Wert festgelegt
	*/
	function MaxLength($intMaxLength = null)
	{
	    if(is_null($intMaxLength))
	    {
	        return $this->intMaxLength;
	    }
	    else
	    {
	        $this->intMaxLength = $intMaxLength;
	    }
	}
	
	/**
	* Legt fest, ob der Inhalt der Spalte gendert werden darf (false = darf geaendert werden, true = darf nicht geaendert werden)
	*
	* get/set-Methode dieses Attributs: {@link ReadOnly() ReadOnly}.
	* @access private
	* @var boolean
	*/
	var $bReadOnly = false;
	
 	/**
	* ReadOnly, ob der Inhalt verndert werden darf (lesen, schreiben)
    *
    * get/set-Methode des Attributs {@link $bReadOnly ReadOnly}.
    * @access public
    * @param boolean optional Wert auf den die Variable ReadOnly gesetzt werden soll.
    * @return boolean aktueller Wert von ReadOnly
    */
    function ReadOnly($bReadOnly = null)
    {
	    if(is_null($bReadOnly))
	    {
	        return $this->bReadOnly;
	    }
	    else
	    {
	        $this->bReadOnly = $bReadOnly;
	    }
    }

	/**
    * DataTable, dem diese DataColumn zugehrt.
    *
    * get-Methode dieses Attributs: {@link DataTable() DataTable}.
    * @access private
    * @var DataTable
    */
  	var $DataTable;

   	/**
   	* das DataTable-Objekt, dem diese DataColumn zugehrt (lesen)
   	*
   	* get-Methode des Attributs DataTable. 
	* @access public
   	* @return DataTable das {@link $DataTable DataTable}, dem diese DataColumn zugehrt.
   	*/
  	 function &DataTable()//&$DataTable)
  	 {
		//if(isset($DataTable))
		//{
		  //  $this->DataTable = &$DataTable;
		//}
		return $this->DataTable;
  	 }
   
   	/**
	* Legt fest, ob der Inhalt dieser Spalte eindeutig sein muss (false = muss nicht eindeutig sein, true = muss eindeutig sein).
	*
	* get/set-Methode dieses Attributs: {@link Unique() Unique}.
	* @access private
	* @var boolean
	*/
	var $bUnique = false;
	
	/**
	* Unique, ob der Inhalt eindeutig sein muss (lesen, schreiben)
    *
    * get/set-Methode des Attributs {@link $bUnique Unique}.
    * @access public
    * @param boolean optional Wert auf den die Variable Unique gesetzt werden soll.
    * @return boolean aktueller Wert von Unique
    */
    function Unique($bUnique = null)
    {
	    if(is_null($bUnique))
	    {
	        return $this->bUnique;
	    }
	    else
	    {
	        $this->bUnique = $bUnique;
	    }
    }
    
    
    
    
    
    /**
    * Gibt an, ob die Spalte Null-Values enthalten darf (ob die Spalte einen Wert besitzen muss oder auch nicht gesetzt sein kann)
    *
    * get/set-Methode dieses Attributs: {@link AllowDBNull() AllowDBNull}.
    * @access private
    * @var boolean
    */
    var $bAllowDBNull = true;
    
    /**
    * get/set-Methode des Attributs {@link $bAllowDBNull AllowDBNull}.
    * @access public
    * @param boolean optional Wert auf den die Variable AllowDBNull gesetzt werden soll.
    * @return boolean aktueller Wert von AllowDBNull
    */
    function AllowDBNull($bAllowDBNull = null)
    {
    	if(is_null($bAllowDBNull))
	{
		return $this->bAllowDBNull;
	}
	else
	{
		$this->bAllowDBNull = $bAllowDBNull;
	}
    }
    
    /**
    * Gibt an, ob der Inhalt dieser Spalte beim Hinzufuegen einer neuen Zeile automatisch erhoeht und eingetragen wird
    * 
    * get/set-Methode dieses Attributs: {@link AutoInkrement() AutoInkrement}
    * @access private
    * @var boolean
    */
    var $bAutoInkrement = false;
    
    /**
    * get/set-Methode des Attributs {@link $bAutoInkrement AutoInkrement}.
    * @access public
    * @param boolean optional Wert auf den die Variable AutoInkrement gesetzt werden soll.
    * @return boolean aktueller Wert von AutoInkrement
    */
    function AutoInkrement($bAutoInkrement = null)
    {
	if(is_null($bAutoInkrement))
	{
		return $this->bAutoInkrement;
	}
	else
	{
		$this->bAutoInkrement = $bAutoInkrement;
	}    
    }
    
    /**
    * Enthaelt den Wert mit dem die Variable {@link $intAutoInkrementSeed AutoInkrementSeed} erhoeht werden soll, wenn die Variable {@link $bAutoInkrement AutoInkrement} true ist.
    *
    * get/set-Methode dieses Attributs {@link AutoInkrementStep() AutoInkrementStep}.
    * @access private
    * @var int 
    */
    var $intAutoInkrementStep = 1;
    
    /**
    * get/set-Methode des Attributs {@link $intAutoInkrementStep AutoInkrementStep}.
    * @access public
    * @param int optional Wert auf den die Variable AutoInkrementStep gesetzt werden soll.
    * @return int aktueller Wert von AutoInkrementStep
    */
    function AutoInkrementStep($intAutoInkrementStep = null)
    {
    	if(is_null($intAutoInkrementStep))
	{
		return $this->intAutoInkrementStep;
	}
	else
	{
		$this->intAutoInkrementStep = $intAutoInkrementStep;
	}  
    }
    
    /**
    * Enthaelt den Start-Wert der Spalte, wenn die Variable {@link $bAutoInkrement AutoInkrement} true ist.
    *
    * get/set-Methode dieses Attributs {@link AutoInkrementSeed() AutoInkrementSeed}.
    * @access private
    * @var int 
    */
    var $intAutoInkrementSeed = 0;
    
    /**
    * get/set-Methode des Attributs {@link $intAutoInkrementSeed AutoInkrementSeed}.
    * @access public
    * @param int optional Wert auf den die Variable AutoInkrementSeed gesetzt werden soll.
    * @return int aktueller Wert von AutoInkrementSeed
    */
    function AutoInkrementSeed($intAutoInkrementSeed = null)
    {
    	if(is_null($intAutoInkrementSeed))
	{
		return $this->intAutoInkrementSeed;
	}
	else
	{
		$this->intAutoInkrementSeed = $intAutoInkrementSeed;
	}
    }    
    
    /**
    * berprft, ob die bergebene Column gleich der aktuellen Column ist
    *
    * @param DataColumn Column mit der auf Gleichheit geprft werden soll
    * @access public
    * @return boolean false = Columns nicht gleich true = Columns gleich
    */
    function Equals($DataColumn)
    {
       	if(!is_null($DataColumn))
		{
		    if($this->strCaption == $DataColumn->strCaption)
		    {
		        if($this->strColumnName == $DataColumn->strColumnName)
		        {
		       		if($this->strDefaultValue == $DataColumn->strDefaultValue)
		        	{
		        		if($this->intMaxLength == $DataColumn->intMaxLength)
		        		{
		        			if($this->bReadOnly == $DataColumn->bReadOnly)
		        			{
								// Hier kann kein einfacher Vergleich
								// $this->DataTable == $DataColumn->DataTable() geschehen, denn 
								// es gibt die zirkulre Referenz 
								// DataTable::Columns[?] <=> DataColumn->DataTable
								// Der bliche PHP-Vergleichsalgorithmus fhrt dann zu einer 
								// endlosen Rekursion. Vergleiche:
								// http://de2.php.net/manual/en/language.oop.object-comparison-php4.php
								// Lsung: nur noch die Namen der DataTables vergleichen
								$yourDataTable = $DataColumn->DataTable();
		        				if($this->DataTable->tableName() == $yourDataTable->tableName())
		        				{
		        					if($this->bUnique == $DataColumn->bUnique)
		        					{
										return true;
		        					}
		        					else
		        					{
		            					return false;
		        					}
		        				}
		        				else
		        				{
		            				return false;
		        				}
		        			}
		        			else
		        			{
		            			return false;
		        			}
		        		}
		        		else
		        		{
		            		return false;
		        		}
		        	}
		        	else
		        	{
		            	return false;
		        	}
		        }
		        else
		        {
		            return false;
		        }
		    }
		    else
		    {
		        return false;
		    }
		}
		else
		{
		    return false;
		}
    }
    
    /**
    * Diese Funktion erstellt eine flache Kopie der DataColumn
    *
    * @access protected
    * @return &DataColumn flache Kopie
    */
    function & MemberwiseClone()
    {
        return $this;
    }
}
?>

