<?php

/**
 * Die Klasse {@link DataRelation}
 *
 * @package phpDataSet
 * @author Susanne Mckl <RezsiaDione@web.de>
 */

/**
 * Fremdschlsselbeziehung zwischen zwei {@link DataTable DataTables} in einem {@link DataSet}
 *
 * @package phpDataSet
 * @author Susanne Mckl <RezsiaDione@web.de>
 */
class DataRelation
{

  /**
   * Erzeugt ein neues Objekt der Klasse DataRelation und initialisiert es.
   *
   * Grundstzlich verbindet ein {@link DataRelation}-Objekt {@link DataColumn}-Objekte, 
   * die zum selben {@link DataSet} gehren. Die DataColumn-Objekte, die die Spalten mit 
   * den Primrschlsselwerten darstellen, haben die Rolle "parent", die anderen 
   * stellen die Spalten mit den Fremdschlsselwerten dar und habene die Rolle 
   * "child". 
   * Ein DataRelation-Objekt verbindet <i>genau</i> zwei Tabellen, daher muss
   * sichergestellt sein, dass alle Child- und Parent-Columns jeweils genau einer
   * DataTable angehoeren. 
   *
   * @param string $name
   * @param DataColumn[] $parentColumn Spalten, von der diese Relation ausgeht.
   * @param DataColumn[] $childColumn Spalten, zu der diese Relation weist.
   */
  function DataRelation(&$parentColumns, &$childColumns, $strRelationName = null)
  {
  	static $objCounter = 0;

    	if(is_null($strRelationName))
	{
      		$this->strRelationName = "DataRelation" . $objCounter;
    	}
    	else
	{
     		$this->strRelationName = $strRelationName;
    	}
    	++$objCounter;
	$this->DataColumnParent = $parentColumns; // = new DataColumn(...);
	$this->DataColumnChild = $childColumns;
  }

	/**
	* Eindeutiger Name der DataRelation
	*
	* get/set-Methode dieses Attributs: {@link RelationName() RelationName}. Wird durch den Konstruktor
    	* erstmals und eindeutig festgelegt.
	* @access private
	* @var string
	*/
	var $strDataRelationName;
	
	/**
    	* Name der DataRelation (lesen, schreiben)
    	*
    	* get/set-Methode des Attributs {@link $strRelationName RelationName}.
    	* @access public
    	* @param string Wert, auf den der Name der DataRelation gesetzt werden soll. Optional.
    	* @return string aktueller Name der DataRelation
    	*/
	function RelationName($strDataRelationName = null)
	{
	    if(is_null($strDataRelationName))
	    {
	        return $this->strRelationName;
	    }
	    else
	    {
	        $this->strRelationName = $strDataRelationName;
	    }
	}
	
	/**
	* DataColumn-Objekt oder Kollektion von diesen, von der die Relation ausgeht.
	*
	* Alle DataColumn-Objekte muessen derselben DataTable angehoeren.
	* get-Methode dieses Attributs: {@link getParentColumn() ParentColumn}.
	* @access private
	* @var DataColumn oder DataColumn[]
	*/
  	var $DataColumnParent;
	
	/**
  	* Diese Funktion liefert die Kollektion DataColumnParent.
  	*
	* get-Methode des Attributs {@link $DataColumnParent ParentColumn}.
  	* @return DataColumn[] 
  	*/
  	function getParentColumns()
  	{
  		return $this->DataColumnParent;
  	}
	
	/**
	* Diese Funktion liefert die DataTable, der die DataColumnParent angehoeren.
	*
	* @return DataTable
	*/
	function getParentTable()
	{
		if(is_array($this->DataColumnParent))
		{
			return $this->DataColumnParent[0]->DataTable();
			//$Tables = array();
			//foreach($this->DataColumnParent as $OneParentColumn)
			//{
			//	$Tables[] = $OneParentColumn->DataTable();
			//}
			//return $Tables;
		}
		else
			return $this->DataColumnParent->DataTable();
	}
	
	/**
	* DataColumn-Objekt oder Kollektion von diesen, zu der diese Relation weist.
	*
	* get-Methode dieses Attributs: {@link getChildColumn() ChildColumn}.
	* @access private
	* @var DataColumn oder []
	*/
  	var $DataColumnChild;
  
  
  	/**
  	* Diese Funktion liefert die Kollektion DataColumnChild.
  	*
	* get-Methode des Attributs {@link $DataColumnChild ChildColumn}.
  	* @return DataColumn[] 
  	*/
  	function getChildColumns()
  	{
  		return $this->DataColumnChild;
  	}
	
	/**
	* Diese Funktion liefert die DataTable, der die DataColumnChild angehoeren.
	*
	* @return DataTable
	*/
	function getChildTable()
	{
		if(is_array($this->DataColumnChild))
		{
			return $this->DataColumnChild[0]->DataTable();
			//$Tables = array();
			//foreach($this->DataColumnChild as $OneChildColumn)
			//{
			//	$Tables[] = $OneChildColumn->DataTable();
			//}
			//return $Tables;
		}
		else
			return 	$this->DataColumnChild->DataTable(); 
	}
	
	/**
    	* DataSet, dem diese DataRelation zugehrt.
    	*
    	* get-Methode dieses Attributs: {@link DataSet() DataSet}.
    	* @access private
    	* @var DataSet
    	*/
  	var $DataSet;

   	/**
   	* das DataSet-Objekt, dem diese DataRelation zugehrt (lesen)
   	*
   	* get-Methode des Attributs {@link $DataSet DataSet}. 
	* @access public
   	* @return DataSet das {@link $DataSet DataSet}, dem diese DataColumn zugehrt.
   	*/
  	 function & DataSet() //(&$DataSet)
  	 {
		//if(isset($DataSet))
		//{
		//    $this->DataSet = &$DataSet;
		//}
		return $this->DataSet;
  	 }
	 
	/**
    * berprft, ob die bergebene Relation gleich der aktuellen Relation ist
    *
    * @param DataRelation Relation mit der auf Gleichheit geprft werden soll
    * @access public
    * @return boolean false = Relations nicht gleich true = Relations gleich
    */
    function Equals($DataRelation)
    {
       	if(!is_null($DataRelation))
	{
	    if($this->strDataRelationName == $DataRelation->strDataRelationName)
	    {
	        if($this->DataColumnChild->Equals($DataRelation->getChildColumn()))
	        {
	       		if($this->DataColumnParent->Equals($DataRelation->getParentColumn()))
	        	{
	        		if($this->DataSet == $DataColumn->DataSet())
	        		{
	        			return true;
	        		}
	        		else
	        		{
	            			return false;
	        		}
	        	}
	        	else
	        	{
	            		return false;
	        	}
	        }
	        else
	        {
	        	return false;
	        }
	    }
	    else
	    {
	    	return false;
	    }
	}
	else
	{
	   return false;
	}
    }
    
	 
	/**
    * Diese Funktion erstellt eine flache Kopie der DataRelation
    *
    * @access protected
    * @return &DataRelation flache Kopie
    */
    function & MemberwiseClone()
    {
        return $this;
    }
}
?>
