<?php

/**
 * Die Klasse {@link DataRowTest}
 *
 * @package phpDataSet
 * @author Andreas Krling <akraeling@t-online.de>
 * @since 20.06.2004
 */
 
/**
 * phpUnit, das hier benutzte Testing-Framework
 *
 * Diese Klasse erwartet das vollstndige Testing-Framework PhpUnit 0.5 im 
 * Verzeichnis ./phpunit . Dies ist auch das Standard-Verzeichnis, in das 
 * sich PhpUnit 0.5 entpackt.
 *
 * PhpUnit 0.5 kann bezogen werden ber {@link http://sourceforge.net/projects/phpunit}. 
 * Hilfreich zum Umgang mit diesem Testing-Framework ist auerdem das 
 * {@link http://sourceforge.net/projects/phpunitcookbook/ PHPUnit Cookbook}
 */
require_once("phpunit/phpunit.php");

require_once("DataRowException.php");
require_once("DataRow.php");
require_once("DataTable.php");

/**
 * Testklasse zur Klasse {@link DataRow}
 *
 * @package phpDataSet
 * @author Andreas Krling <akraeling@t-online.de>
 * @since 20.06.2004
 */ 
class DataRowTest extends TestCase{

  var $Row;
  var $DTable;
  
  function setUp(){ 
    $this->DTable = & new DataTable("DataTab_Test");      
    
    $myDataTableSchema   = & $this->DTable->columns();
    $myDataTableSchema[] = new DataColumn(& $this->DTable, "Name");
    $myDataTableSchema[] = new DataColumn(& $this->DTable, "Vorname");
    $myDataTableSchema[] = new DataColumn(& $this->DTable, "Alter");   
    
    $testArr = array(0 =>"Kraeling",1 =>"Andreas", 2 =>678801);
    $this->Row =& new DataRow(& $this->DTable, $testArr);
   }  
   
  /**
   * Testen der Fkt. {@link DataRow::itemArray()}; 
   */  
  function test_itemArray(){
    $this->assertEquals(
      array("Kraeling", "Andreas", 678801),   //erwarteter Wert
      $this->Row->ItemArray,                  //aktueller wert
      "Fehler bla"                            //Fehlermeldung
    );
  }

  /**
   * Testen der Fkt. {@link DataRow::rowState()};  
   */
  function test_rowState(){ 
     $this->Row->rowState(true);
     $this->assertEquals(
       true,
       $this->Row->rowState(),
       "Geht net"
     );
   }

   /**
    * Testen der Fkt. {@link DataRow::item()}; 
    */
  function test_item(){
    //$this->Row->item("Kraeling");
    $this->assertEquals(
      "Kraeling",
      $this->Row->item(0),
      "Geht net"
    );
       
    $this->assertEquals(
      678801,
      $this->Row->item(2),
     "Geht net"
    );
  }
   
   /**
    * Testen der Fkt. {@link DataRow::table()}
    */
  function test_table(){
    // Vergleichen ist mit den PHP-internen Funktionen bei zirkulren Referenzen unmglich. 
    // Deshalb werden diese entfernt. Alternativ sollte DataTable::Equals() implementiert werden.
    $tmp =& $this->DTable->columns();
    $tmp = null;
    $tmp =& $this->Row->table();
    $tmp =& $tmp->columns();
    $tmp = null;
           
    $this->assertEquals(
      $this->DTable,
      $this->Row->table(),
      "Geht net"
    );
  }

  /**
   *  Testen der Fkt. {@link DataRow::Equals()}; 
   */
  function test_equals(){
       
    //Anlegen eines DataRow-Objects zum Vergleich mit dem aktuellen $this->Row
    $testArr = array(0 =>"Kraeling",1 =>"Andreas", 2 =>678801);
    $secondRow =& new DataRow(& $this->DTable, $testArr);

    $this->assertEquals(
      true,
      $this->Row->Equals($secondRow),
      "Geht net"
    );
  }
}
?>
