<?php

/**
 * Die Klasse {@link DataSetTest}
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */
 
/**
 * PhpUnit, das hier benutzte Testing-Framework
 *
 * Diese Klasse erwartet das vollstndige Testing-Framework PhpUnit 0.5 im 
 * Verzeichnis ./phpunit . Dies ist auch das Standard-Verzeichnis, in das 
 * sich PhpUnit 0.5 entpackt.
 *
 * PhpUnit 0.5 kann bezogen werden ber {@link http://sourceforge.net/projects/phpunit}. 
 * Hilfreich zum Umgang mit diesem Testing-Framework ist auerdem das 
 * {@link http://sourceforge.net/projects/phpunitcookbook/ PHPUnit Cookbook}
 */
require_once("phpunit/phpunit.php");

require_once("DataSet.php");
require_once("DataTable.php");

/**
 * Test der Klasse {@link DataSet}
 *
 * Diese Klasse ist gedacht zur Verwendung mit dem 
 * Test-Framework {@link http://phpunit.sourceforge.net PhpUnit}.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 * @todo Test der Methode {@link DataSet::Merge()} mit {@link MissingSchemaAction::Error()} und 
 *   {@link MissingSchemaAction::AddWithKey()}.
 * @todo Test der Methode {@link DataSet::Merge()} unter Verwendung eines {@link DataSet}
 */
class DataSetTest extends TestCase {

  /**
   * Das {@link DataSet DataSet}, auf dem die test-Methoden hauptschlich operieren
   *
   * @var DataSet
   */
  var $myDataSet;
  
  /**
   * Enthlt ein {@link DataSet} im Originalzustand von {@link myDataSet}, das heit den 
   * Zustand nach Ausfhrung des Konstruktors.
   *
   * @var DataSet
   */
  var $myOriginalDataSet;
  
  /**
   * Ein {@link DataSet DataSet} mit Tabellen, die zulssige und unzulssige Schemaerweiterungen 
   * der Tabellen in {@link myDataSet} sind.
   *
   * @var DataSet
   */
  var $myDataSetExt;

  /**
   * Erzeuge Objekte, auf denen alle Tests operieren sollen.
   *
   * berschriebene Methode der Klasse {@link TestCase}, die zum 
   * Test-Framework {@link http://phpunit.sourceforge.net PhpUnit} gehrt. Es werden die 
   * Attribute des aktuellen Objektes so gesetzt, dass sie vorbereitet fr die Ausfhrung des 
   * nchsten Tests sind. Ein Test besteht in der Ausfhrung einer Methode, die mit test
   * beginnt.
   *
   * @access public
   */
  function setUp() {
    // setUp() muss jeder Testmethode eine identische Umgebung bereitstellen. Dazu gehrt auch, 
    // dass statische Variablen denselben Wert haben:
    DataSet::presetObjCounter(0);

    // erste DataTable, die spter in das DataSet aufgenommen wird
    $myDataTable1 =& new DataTable("Altersangaben1");
    $myDataTable1Schema =& $myDataTable1->columns();
    $myDataTable1Schema[] = new DataColumn(& $myDataTable1, "Name");
    $myDataTable1Schema[] = new DataColumn(& $myDataTable1, "Vorname");
    $myDataTable1Schema[] = new DataColumn(& $myDataTable1, "Alter");
    $myDataTable1->primaryKey(array(& $myDataTable1Schema[0], & $myDataTable1Schema[1]));
    $myDataTable1Rows =& $myDataTable1->rows();
    $myDataTable1Rows[] = new DataRow(& $myDataTable1, array("Seeler", "Holger", 20));
    $myDataTable1Rows[] = new DataRow(& $myDataTable1, array("Seele",  "Holge",  21));
    $myDataTable1Rows[] = new DataRow(& $myDataTable1, array("Seel",   "Holg",   20));
    $myDataTable1Rows[] = new DataRow(& $myDataTable1, array("See",    "Hol",    21));
    $myDataTable1Rows[] = new DataRow(& $myDataTable1, array("Se",     "Ho",     22));
    
    // zweite DataTable, die spter in das DataSet aufgenommen wird
    $myDataTable2 =& new DataTable("Altersangaben2");
    $myDataTable2Schema =& $myDataTable2->columns();
    $myDataTable2Schema[] = new DataColumn(& $myDataTable2, "Name");
    $myDataTable2Schema[] = new DataColumn(& $myDataTable2, "Vorname");
    $myDataTable2Schema[] = new DataColumn(& $myDataTable2, "Alter");
    $myDataTable2->primaryKey(array(& $myDataTable2Schema[0], & $myDataTable2Schema[1]));
    $myDataTable2Rows =& $myDataTable2->rows();
    $myDataTable2Rows[] = new DataRow(& $myDataTable2, array("Seeler", "Holger", 20));
    $myDataTable2Rows[] = new DataRow(& $myDataTable2, array("Seele",  "Holge",  21));
    $myDataTable2Rows[] = new DataRow(& $myDataTable2, array("Seel",   "Holg",   20));
    $myDataTable2Rows[] = new DataRow(& $myDataTable2, array("See",    "Hol",    21));
    $myDataTable2Rows[] = new DataRow(& $myDataTable2, array("Se",     "Ho",     22));
    
    // DataSet erzeugen, mit dem die test-Methoden arbeiten drfen
    $this->myDataSet = new DataSet("myDataSet");
    // DataSet::Reset() soll eine DataTable in ihren Originalzustand (d.i. ihr Zustand nach 
    // Konstruktoraufruf) zurckversetzen. Erhalte also den Originalzustand zum Vergleichen:
    $this->myOriginalDataSet = $this->myDataSet;
    
    // erzeugte DataTables in das DataSet aufnehmen
    $myDataSetTables =& $this->myDataSet->tables();
    $myDataSetTables[$myDataTable1->tableName()] = $myDataTable1;
    $myDataSetTables[$myDataTable2->tableName()] = $myDataTable2;
    
    // Tabelle, die eine zulssige Schemaerweiterung zu $this->myDataSet darstellt:
    $myDataTable1Ext =& new DataTable("Altersangaben1");
    $myDataTable1ExtSchema =& $myDataTable1Ext->columns();
    $myDataTable1ExtSchema = $myDataTable1Schema;
    $myDataTable1ExtSchema[] = new DataColumn(& $myDataTable1Ext, "Geburtstag");
    $myDataTable1ExtRows =& $myDataTable1Ext->rows();
    $myDataTable1ExtRows[] = new DataRow(& $myDataTable1Ext, array("Prophet", "Hulda", 20, "10d"));
    $myDataTable1ExtRows[] = new DataRow(& $myDataTable1Ext, array("Prophe",  "Huld",  21, "20d"));
    $myDataTable1ExtRows[] = new DataRow(& $myDataTable1Ext, array("Proph",   "Hul",   20, "30d"));
    $myDataTable1ExtRows[] = new DataRow(& $myDataTable1Ext, array("Prop",    "Hu",    21, "40d"));
    $myDataTable1ExtRows[] = new DataRow(& $myDataTable1Ext, array("Pro",     "H",     22, "50d"));

    // Tabelle, die eine unzulssige Schemaerweiterung zu $this->myDataSet darstellt:    
    $myDataTable2NoExt =& new DataTable("Altersangaben2");
    $myDataTable2NoExtSchema =& $myDataTable2NoExt->columns();
    unset($myDataTable2NoExtSchema[0]);
    $myDataTable2NoExtSchema[] = new DataColumn(& $myDataTable2NoExt, "Geburtstag");
    $myDataTable2NoExtSchema[] = new DataColumn(& $myDataTable2NoExt, "Todestag");
    
    // DataSet erzeugen, das die zulssigen und unzulssigen Schemaerweiterungen aufnimmt:
    $this->myDataSetExt = new DataSet("myDataSetExt");

    // erzeugte DataTables in das DataSet aufnehmen
    $myDataSetExtTables =& $this->myDataSetExt->tables();
    $myDataSetExtTables[$myDataTable1Ext->tableName()] = $myDataTable1Ext;
    $myDataSetExtTables[$myDataTable2NoExt->tableName()] = $myDataTable2NoExt;
  }

  /**
   * Ressourcenfreigabe permanent allozierter Ressourcen nach Durchfhrung eines Tests.
   *
   * berschriebene Methode der Klasse {@link TestCase}, die zum 
   * Test-Framework {@link http://phpunit.sourceforge.net PhpUnit} gehrt. Es werden die 
   * Attribute des aktuellen Objektes so gesetzt, dass alle fr die Durchfhrung eines Tests 
   * belegten Ressourcen wieder freigegeben sind. Ein Test besteht in der Ausfhrung einer 
   * Methode, die mit test beginnt.
   *
   * @access public
   */
  function teardown() {
  }
  
  /**
   * Testet die Methode {@link DataSet::Clear() DataSet::Clear()}
   */
  function test_Clear() {
    $this->myDataSet->Clear();
    foreach ($this->myDataSet->tables() as $name => $oneOfMyTables) {
      $this->assertEquals(
        null,
        $oneOfMyTables->rows(),
        "die DataRow-Kollektion der Tabelle $name ist nicht null"
      );
    }
  }
  
  /**
   * Testet ob das Attribut {@link DataSet::$DataSetName DataSet::$DataSetName} korrekt 
   * verwaltet wird.
   */
  function test_DataSetName() {
    $DSetAutoid = new DataSet();
    $DSetMyname = new DataSet("Myname");

    // Erwarte Name "DataSet2" statt "DataSet0", weil bereits im automatisch vor dieser Methode 
    // aufgerufenen $this->setUp() bereits zwei andere DataSets erzeugt wurden.
    $this->assertEquals(
      "DataSet2",
       $DSetAutoid->dataSetName(),
      "automatisch generierte DataSet-Namen werden nicht korrekt generiert, gespeichert " .
      "oder ausgelesen"
    );

    $this->assertEquals(
      "Myname",
      $DSetMyname->dataSetName(),
      "benutzerdefinierte DataSet-Namen werden nicht korrekt abgespeichert oder ausgelesen"
    );
  }

  /**
   * Testet die Methode {@link DataSet::Reset() DataSet::Reset()}
   */  
  function test_Reset() {
    $this->myDataSet->Reset();

    $this->assertEquals(
      $this->myOriginalDataSet,
      $this->myDataSet,
      "das DataSet wurde nicht in seinen Zustand nach Konstruktoraufruf zurckgesetzt"
    );
  }
  
  /**
   * Testet das Attribut {@link DataSet::$Tables DataSet::$Tables}
   */
  function test_Tables() {
    // Der Test, ob $this->myDataSet die nach $this->setUp() erwarteten DataTables enthlt, 
    // zeigt auch, dass DataTables zum DataSet hinzugefgt werden knnen; denn das wurde in 
    // $this->setUp() mit den jetzt enthaltenen DataTables gemacht:
    $myDataSetTables =& $this->myDataSet->tables();
    $this->assertEquals(
      "Altersangaben1",
      $myDataSetTables["Altersangaben1"]->tableName(),
      "Tabelle \"Altersangaben1\" wurde nicht korrekt gespeichert"
    );
    $this->assertEquals(
      "Altersangaben2",
      $myDataSetTables["Altersangaben2"]->tableName(),
      "Tabelle \"Altersangaben2\" wurde nicht korrekt gespeichert"
    );
    
    //Test, ob Tabellen gelscht werden knnen:
    unset($myDataSetTables["Altersangaben1"]);
    $this->assertEquals(
      array(0 => "Altersangaben2"),
      array_keys($myDataSetTables),
      "Tabelle \$myDataSetTables[\"Altersangaben1\"] konnte nicht gelscht werden"
    );
    $this->assert(
      !array_key_exists("Altersangaben1", $myDataSetTables),
      "Tabelle \$myDataSetTables[\"Altersangaben1\"] konnte nicht gelscht werden"
    );
  }
  
  /**
   * Testet die Methode {@link DataSet::containsBasicSchemaOf() DataSet::containsBasicSchemaOf()}
   */
  function test_containsBasicSchemaOf() {

    // zulssige und unzulssige Erweiterungen extrahieren
    // $this->myDataSetExt selbst ist eine unzulssige Erweiterung, weil es eine DataTable enthlt, 
    // die eine unzulssige Erweiterung ist.
    $myDataSetNoExt =& $this->myDataSetExt;
    $myDataSetNoExtTables =& $myDataSetNoExt->tables();
    $myDataTable1Ext = $myDataSetNoExtTables["Altersangaben1"];
    $myDataTable2NoExt = $myDataSetNoExtTables["Altersangaben2"];

    $this->assertEquals(
      false,
      $this->myDataSet->containsBasicSchemaOf($myDataSetNoExt),
      "unzulssige Schemaerweiterung durch ein DataSet nicht erkannt"
    );

    $this->assertEquals(
      true,
      $this->myDataSet->containsBasicSchemaOf($myDataTable1Ext),
      "zulssige Schemaerweiterung durch eine DataTable nicht erkannt"
    );
    
    $this->assertEquals(
      false,
      $this->myDataSet->containsBasicSchemaOf($myDataTable2NoExt),
      "unzulssige Schemaerweiterung durch eine DataTable nicht erkannt"
    );
  }
  
  /**
   * Testet die Methode {@link DataSet::Merge() DataSet::Merge()} durch Zusammenfhren mit 
   * einer {@link DataTable DataTable}.
   */
  function test_Merge_DataTable() {
    
    // Merge mit einer DataTable, deren Schema eine zulssige Erweiterung ist:
    
    // Quelltabelle ermitteln, d.i. diejenige die ihre Daten zur Zusammenfhrung bereitstellt:
    $mergingTableColl =& $this->myDataSetExt->tables();
    $mergingTable =& $mergingTableColl["Altersangaben1"];
    
    // Zieltabelle ermitteln, d.i. diejenige die um die Daten erweitert werden wird:
    $mergeeTableColl =& $this->myDataSet->tables();
    $mergeeTable =& $mergeeTableColl["Altersangaben1"];
    
    $signalQueue = array();
    
    $this->myDataSet->Merge($mergingTable, $signalQueue, MissingSchemaAction::Add());
    $this->assertEquals(
      array(),
      $signalQueue,
      "Merge mit einer Tabelle: zulssige Schemaerweiterung wurde nicht erkannt"
    );
    $this->assertEquals(
      10,
      count($mergeeTable->rows()),
      "Merge mit einer Tabelle mit zulssiger Schemaerweiterung fhrte nicht zur Aufnahme aller " .
        "neuen Zeilen"
    );
    $this->assertEquals(
      4,
      count($mergeeTable->columns()),
      "Merge mit einer Tabelle mit zulssiger Schemaerweiterung und MissingSchemaAction::Add() " .
        "fhrte nicht zur erwarteten Erweiterung auf das neue Schema"
    );
  }
  
  /**
   * Testet die Methode {@link DataSet::Merge() DataSet::Merge()} durch Zusammenfhren mit 
   * einem Array von {@link DataRow DataRows}.
   */
  function test_Merge_DataRowColl() {
  
    // Merge mit DataRow[], deren Schema eine zulssige Erweiterung ist:

    // Quelltabelle ermitteln, d.i. diejenige die ihre Daten zur Zusammenfhrung bereitstellt:
    $mergingTableColl =& $this->myDataSetExt->tables();
    $mergingTable =& $mergingTableColl["Altersangaben1"];
    $mergingRows = $mergingTable->rows();
    
    // Zieltabelle ermitteln, d.i. diejenige die um die Daten erweitert werden wird:
    $mergeeTableColl =& $this->myDataSet->tables();
    $mergeeTable =& $mergeeTableColl["Altersangaben1"];

    $signalQueue = array();
    
    $this->myDataSet->Merge($mergingRows, $signalQueue, MissingSchemaAction::Ignore());
    $this->assertEquals(
      array(),
      $signalQueue,
      "Merge mit DataRow[]: zulssige Schemaerweiterung wurde nicht erkannt"
    );
    $this->assertEquals(
      10,
      count($mergeeTable->rows()),
      "Merge mit einem DataRow[] mit zulssiger Schemaerweiterung fhrte nicht zur Aufnahme " .
        "aller neuen Zeilen"
    );
    $this->assertEquals(
      3,
      count($mergeeTable->columns()),
      "Merge mit einem DataRow[] mit zulssiger Schemaerweiterung und " .
        "MissingSchemaAction::Ignore() fhrte nicht zur erwarteten Vernachlssigung der " .
        "zustzlichen Spalten"
    );
  }
  
  /**
   * Testet die Methode {@link DataSet::makeTyped() DataSet::makeTyped()}
   */
  function test_makeTyped() {

    $this->myDataSet->makeTyped();

    $this->assertEquals(
      //get_class() ermittelt den Klassennamen in Kleinbuchstaben, deshalb erwarten wir 
      //  hier "dataset_mydataset" und nicht den wirklichen Namen "DataSet_myDataSet":
      "dataset_mydataset",
      get_class($this->myDataSet),
      "Der Klassenname des DataSet nach der Typisierung entspricht nicht dem DataSet-Namen "
        . "mit vorangestelltem \"DataSet_\""
    );
    
    /* Solange noch nicht DataTable::equals() zur Verfgung steht, mssen DataTable-Objekte in 
     * serialisierter Form verglichen werden. Jeder direkte Vergleich von DataTable-Objekten durch 
     * PHP ist unmglich aufgrund zirkulrer Referenzen
     */

    $this->assertEquals(
      serialize($this->myDataSet->Altersangaben1()),
      serialize($this->myDataSet->tables("Altersangaben1")),
      "Die get-Methode myDataSet::Altersangaben1() greift nicht auf die zugehrige Tabelle "
        . "myDataSet::tables(\"Altersangaben1\") zu."
    );
    
    $this->assertEquals(
      serialize($this->myDataSet->Altersangaben2()),
      serialize($this->myDataSet->tables("Altersangaben2")),
      "Die get-Methode myDataSet::Altersangaben2() greift nicht auf die zugehrige Tabelle "
        . "myDataSet::tables(\"Altersangaben2\") zu."
    );
  }

}

?>
