<?php

/**
 * Die Klasse {@link DataTable}
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 * @todo Codegenerator vollstndig implementieren. Bisher implementiert ist 
 *   {@link DataSet::makeTyped()} und {@link DataSet::extend()}, so dass ansatzweise 
 *   gezeigt wird, wie das in einem beiliegenden Dokument vorgestellte Design eines 
 *   Codegenerators fr phpDataSet umgesetzt werden kann. (Nach Version 1.0).
 * @todo Implementierung der Klasse Constraint. (Nach Version 1.0).
 * @todo Konventionen fr DocBlocks festlegen und umsetzen (nach Version 1.0):
 *   [Hinweis: Es ist nicht mglich, {@link} fr den Rckgabewert von Funktionen zu verwenden.]
 *   <ul>
 *     <li>Welche Tags werden verwendet?</li>
 *     <li>Wann werden Links gesetzt?</li>
 *     <li>Wie wird eingerckt?</li>
 *     <li>Wie und wann werden Abstze geschaltet?</li>
 *     <li>Wie sieht die Beschreibung von Links zu Variablen (mit "$" ?) und Funktionen (mit 
 *       "()" ?) aus?</li>
 *     <li>Welche Anfhrungszeichen werden verwendet?</li>
 *     <li>Wo werden @todo-Tags geschrieben, die sich auf Klassen anderer Autoren beziehen? An der 
 *       Stelle, an der die Aufgabe besteht, also in der Klasse der anderen Autoren! Alternativ 
 *       knnen diese @todo-Eintrge in die Dokumentation der Datei aufgenommen werden, 
 *       ebenso @todo-Eintrge die sich auf keine konkrete Klasse beziehen.</li>
 *     <li>Wie werden ganze Klassen dokumentiert?</li>
 *     <li>Sind Verweise auf Attribute zulssig (nein, sie sind ja private!)?</li>
 *   </ul>
 *
 * @todo Durchsetzung von Constraints in {@link DataTable} implementieren (nach Version 1.0). 
 *   Dazu sind ntig:
 *   <ul>
 *     <li>Reaktion auf Einfgen von {@link DataRow DataRows} in das Attribut {@link Rows}. Um 
 *       eine solche Reaktion zu ermglichen muss evtl. der set-Zugriff auf dieses Attribut 
 *       umgestellt werden: nicht mehr direkter Zugriff auf das Array, sondern mit Methoden 
 *       wie add, delete usw.. Dazu muss evtl. doch noch eine Klasse DataRowCollection 
 *       definiert werden. Der Einheitlichkeit halber muss der Zugriff auf {@link DataColumns},
 *       {@link DataRelations} und und Constraints dann analog ablaufen. In Microsoft .NET 
 *       besteht diese Reaktion durch eine ConstraintException (siehe
 *       {@link http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpref/html/frlrfsystemdatadatarowcollectionclassaddtopic1.asp
         System.Data.RowCollextion (MSDN)}).</li>
 *     <li>nderungen an den Methoden  {@link DataTable::LoadDataRow()} 
 *       und {@link DataTable::EndLoadData()}: abhngig von {@link DataTable::$EnforceConstraints} 
 *       sollen Constraints bei jedem Aufruf von {@link DataTable::LoadDataRow()} oder 
 *       nur am Ende aller Aufrufe (also in {@link DataTable::EndLoadData()}) geprft werden. 
 *       In Microsoft .NET ist dieses Verhalten durch eventuelle Generierung einer 
 *       ConstraintException implementiert.</li>
 *   </ul>
 */

/**
 * Tabelle mit Daten, im Hauptspeicher gehalten, am relationalen Datenmodell orientiert
 *
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 * @package phpDataSet
 */
class DataTable {

  /**
   * intitalisiert eine neue Instanz eines {@link DataTable}-Objekts
   *
   * @access public
   * @param string $TableName optionaler Name, den die Tabelle haben soll. Wird er 
   *   angegeben, so muss er anwendungsweit eindeutig sein. Wird er nicht angegeben, so wird 
   *   im Konstruktor ein anwendungsweit eindeutiger Name generiert.
   */
  function DataTable($TableName = null) {
    static $objCounter = 0;
    // Startwert auswerten und durch "null" gleichzeitig zurcksetzen:
    $presetValue = $this->presetObjCounter(null);
    if (isset($presetValue)) {
      $objCounter = $presetValue;
    }

    if (is_null($TableName)) {
      $this->tableName("DataTable" . $objCounter);
    }
    else {
      $this->tableName($TableName);
    }
    
    ++$objCounter;
  }
  
  /**
   * Klassenweiter Startwert fr einen Zhler, der zur Generierung eindeutiger Namen 
   * fr {@link DataTable}-Objekte im Konstruktor verwendet wird.
   *
   * Der Startwert wird beim nchsten Aufruf von {@link DataTable::DataTable()} ausgewertet und 
   * dabei gleichzeitig gelscht, so dass folgende Aufrufe von {@link DataTable::DataTable()} 
   * beim aktuellen Zhlerstand und nicht wieder bei diesem Startwert beginnen. Lschen 
   * bedeutet, dass dieser Startwert auf null gesetzt wird.
   *
   * Zugriff eigentlich friend, erlaubt auch fr DataTableTest. Gedacht ist diese Methode 
   * dazu, in Tests auch statische Variablen direkt setzen zu knnen, um so ihre aktuellen 
   * Werte zu kennen und sinnvolle Annahmen ber zuknftige Werte in den Tests machen zu 
   * knnen.
   *
   * @access private
   * @param mixed $newPresetValue neuer Wert, den der Startwert annehmen soll. Es kann ein 
   *   Integer oder <kbd>null</kbd> bergeben werden. Letzteres bedeutet ein Lschen des 
   *   Startwertes.
   * @return mixed bisheriger Wert des Startwertes. Es kann ein Integer sein oder <kbd>null</kbd>.
   *   Der letzte Fall bedeutet, dass der Startwert nicht gesetzt ist.
   */
  function presetObjCounter($newPresetValue) {
    static $presetValue;
    $tmp = $presetValue;
    $presetValue = $newPresetValue;
    return $tmp;
  }

  /**
   * Name dieser {@link DataTable}
   *
   * get/set-Methode dieses Attributs: {@link tableName}. Wird durch den Konstruktor 
   * erstmals und eindeutig festgelegt.
   * @access private
   * @var string
   */
  var $TableName;
  
  /**
   * Name der {@link DataTable} (lesen, schreiben)
   *
   * get/set-Methode des Attributs {@link TableName}.
   *
   * @access public
   * @param string Wert, auf den der Name der {@link DataTable} gesetzt werden soll. Optional.
   * @return string aktueller Name der {@link DataTable}
   */
  function tableName($TableName = null) {
    if (!is_null($TableName)) {
      $this->TableName = $TableName;
    }
    return $this->TableName;
  }
  
  /**
   * Das {@link DataSet}, dem diese {@link DataTable} zugehrt
   * 
   * get/set-Methode dieses Attributs: {@link dataSet}. Standardwert: <kbd>null</kbd>.
   *
   * @access private
   * @var DataSet
   */
  var $DataSet;
  
  /**
   * Eine Variable mit dem Wert <kbd>null</kbd>.
   *
   * Dieses Attribut wird nur fr interne Zwecke verwendet: um nichts bergeben zu knnen, wo 
   * eine Methode eine Referenz auf ein {@link DataSet} erwartet. Denn in PHP kann nicht dort, 
   * wo eine Referenz erwartet wird, direkt <kbd>null</kbd> bergeben werden.
   *
   * @access private
   * @var null
   */
  var $nullDataSet = null;
  
  /**
   * das {@link DataSet}, dem diese {@link DataTable} zugehrt (lesen, schreiben)
   *
   * get/set-Methode des Attributs {@link $DataSet}. Zugriff auf die set-Methode nur von Instanzen 
   * der Klasse {@link DataSet}. Leider kann friend-Semantik nicht in PHP4-Syntax ausgedrckt 
   * werden.
   *
   * Eine {@link DataTable} kann unabhngig existieren: dann liefert diese Methode den 
   * Wert <kbd>null</kbd>. Eine {@link DataTable} kann aber auch Bestandteil genau 
   * eines {@link DataSet DataSets} sein. Dann liefert diese Methode dieses {@link DataSet}.
   *
   * @access public
   * @return DataSet das {@link DataSet}, dem diese {@link DataTable} zugehrt.
   * @param DataSet $DataSet Referenz auf das {@link DataSet}, dem diese {@link DataTable} von nun an 
   *   zugehren soll. Um diese {@link DataTable} wieder unabhngig zu machen, bergebe man <kbd>null</kbd>.
   *   Optionales Argument, wird nur beim Einsatz dieser Methode als set-Methode gebraucht.
   *   Weil dieses Argument mit Standardwerten arbeitet und eine Verwendung von Referenzparametern 
   *   zusammen mit Standardwerten unmglich ist, muss der Aufrufer selbst das Referenzzeichen an 
   *   den bergebenen Parameter anfgen statt dass dies von der Methodenschnittstelle bernommen 
   *   wird.
   * @todo Es ist nicht mglich, eine {@link DataTable}, die einem {@link DataSet} zugehrt, wieder 
   *   unabhngig zu machen. Denn wenn <kbd>null</kbd> als Argument bergeben wird (genauer: eine 
   *   Referenz auf eine Variable, die den wert <kbd>null</kbd> hat), so wird das so interpretiert, 
   *   dass die Methode nur als get-Methode eingesetzt wird.
   */ 
  function & dataSet($DataSet = 0) {
    //
    
    if (!is_null($DataSet)) {
      $this->DataSet = &$DataSet;
    }
    return $this->DataSet;
  }
  
  /**
   * alle Spalten dieser Tabelle in Form einer Kollektion von {@link DataColumn}-Objekten
   *
   * get/set-Methode dieses Attributs: {@link columns}. Standardwert: <kbd>null</kbd>
   *
   * @access private
   * @var DataColumn[]
   */
  var $Columns;
  
  /**
   * alle Spalten dieser Tabelle in Form einer Kollektion 
   * von {@link DataColumn}-Objekten (lesen, schreiben).
   * 
   * get/set-Methode des Attributs {@link Columns}. Schreiben ist durch ndern des Rckgabewertes 
   * mglich. Dieses Attribut ist ein numerisches Array. Die Menge aller Spalten 
   * einer {@link DataTable} nennt man ihr Schema. Es definiert die Datenstruktur der Zeilen dieser 
   * Tabelle, d.i. ihrer Datenstze.
   *
   * @access public
   * @return &DataColumn[] Kollektion aller Spalten dieser Tabelle
   */
  function & columns() {
    return $this->Columns;
  }
  
  /**
   * der Primrschlssel dieser {@link DataTable} (lesen, schreiben)
   * 
   * get/set-Methode dieses Attributs: {@link primaryKey}
   *
   * @access private
   * @var DataColumn[]
   */
  var $PrimaryKey;

  /**
   * Primrschlssel dieser DataTable (lesen, schreiben)
   *
   * get/set-Methode des Attributs {@link PrimaryKey}. Standardwert: <kbd>null</kbd>.
   * Weil der Primrschlssel auch aus mehr als einer {@link DataColumn} bestehen kann, wird er 
   * durch ein Feld von {@link DataColumn DataColums} dargestellt.
   *
   * @access public
   * @param DataColumn[] $PrimaryKey Spalten, die den neuen Primrschlssel bilden sollen. Zu 
   *   bergeben als Feld von Referenzen auf {@link DataColumn}-Objekte, die im 
   *   Attribut {@link Columns} enthalten sind. Parameter ist optional und wird nur beim Einsatz 
   *   als set-Methode bentigt.
   * @return DataColumn[] aktueller Wert des Primrschlssels. Die Elemente des Feldes sind 
   *   Referenzen auf DataColumn-Objekte des Attributs {@link Columns}.
   */
  function primaryKey($PrimaryKey = null) {
    if (!is_null($PrimaryKey)) {
      $this->PrimaryKey = $PrimaryKey;
    }
    return $this->PrimaryKey;
  }
  
  /**
   * Kollektion aller Zeilen dieser {@link DataTable}
   *
   * get/set-Methode dieses Attributs: {@link rows}. Standardwert: <kbd>null</kbd>.
   *
   * @access private
   * @var DataRow[]
   */
   var $Rows;
  
  /**
   * Kollektion aller Zeilen, die zu dieser {@link DataTable} gehren (lesen, schreiben)
   * 
   * get/set-Methode des Attributs {@link Rows}. Schreiben ist durch nderung des Rckgabewertes 
   * mglich, er ist eine Referenz. Dieses Attribut ist dabei als numerisch indiziertes 
   * PHP-Array zu behandeln, Spalten werden also z.B. hinzugefgt durch 
   * <kbd>$myRows[] = $newDataRow</kbd>. {@link DataRow DataRows} mssen vor dem 
   * Hinzufgen mit {@link NewRow} erzeugt werden.
   *
   * @access public
   * @return &DataRow[] Kollektion aller Zeilen, die zu dieser {@link DataTable} gehren
   */
  function & rows() {
    return $this->Rows;
  }
  
  /**
   * lscht alle Daten der {@link DataTable}
   *
   * Alle Zeilen dieser Tabelle werden entfernt. Das Schema der Tabelle und die Beziehungen 
   * zu anderen Tabellen bleiben jedoch erhalten. Das heit, diese Methode ndert nichts 
   * an {@link DataRelation}-Objekten, {@link DataColumn}-Objekten oder Constraint-Objekten.
   *
   * @access public
   */
  function Clear() {
    $this->Rows = null;
  }
  
  /**
   * Kopiert ein {@link DataRow}-Objekt in diese {@link DataTable}, sein Zustand bleibt erhalten
   *
   * Das bergebene {@link DataRow}-Objekt wird an die {@link DataRow}-Kollektion 
   * {@link Rows} dieser {@link DataTable} angehngt. Dabei werden keine Constraints 
   * bercksichtigt, diese Methode garantiert auch keine eindeutigen Primrschlsselwerte in 
   * {@link Rows} als Nachbedingung. Das kopierte DataRow-Objekt hat denselben Zustand wie das 
   * ursprngliche: alle Eigenschaften und Werte haben denselben Wert.
   *
   * @access public
   * @param DataRow $DataRow Das {@link DataRow}-Objekt, das an die {@link DataRow}-Kollektion 
   *   (d.i. an {@link Rows}) dieser {@link DataTable} angehngt werden soll.
   */
  function ImportRow($DataRow) {
    //es ist nicht mglich, auf die von einer Funktion per Referenz zurckgegebene Variable 
    //mit Hilfe des Funktionsaufrufs direkt zuzugreifen: $this->rows()[] = $DataRow; ist nicht 
    //mglich, es muss eine Zwischenvariable verwendet werden wie hier:
    $Rows =& $this->rows();
    $Rows[] = $DataRow; // $DataRow an das autoindizierte Feld $Rows anhngen.
  }
  
  /**
   * sucht und aktualisiert eine spezifizierte Zeile
   *
   * Die im Attribut {@link PrimaryKey} als Primrschlssel festgelegten Spalten der bergebenen 
   * Werte werden untersucht. Existiert in der {@link DataTable} eine Zeile mit diesen 
   * Primrschlsselwerten, so wird sie aktualisiert und <kbd>null</kbd> wird zurckgegeben. 
   * Existiert noch keine solche Zeile, so wird sie in der {@link DataTable} erzeugt und 
   * gleichzeitig eine Referenz darauf zurckgegeben.
   *
   * @access public
   * @param mixed[] $values Feld der Werte, die in die betreffende Zeile geladen werden sollen. Die 
   *   Reihenfolge der Werte muss der Reihenfolge der Spalten in {@link Columns} entsprechen, 
   *   ebenso die Anzahl der Werte. Fr SQL-NULL ist <kbd>null</kbd> anzugeben. Hat eine Spalte einen 
   *   Standardwert und soll er verwendet werden, so ist ebenfalls <kbd>null</kbd> anzugeben.
   * @return DataRow Wenn eine Zeile aktualisiert wurde: null. Wenn eine neue Zeile erzeugt
   *   wurde: eine Referenz auf die neue Zeile.
   * @todo Exception-Behandlung entsprechend dem .NET-Vorbild dieser Methode implementieren (nach 
   *   Version 1.0). Dabei sind folgende vier Exceptions unter jeweils spezifischen Voraussetzungen 
   *   zu implementieren:
   *   <ul>
   *     <li>ArgumentException: wenn das bergebene Feld mehr Werte als diese Tabelle Spalten hat. 
   *       Kann bereits jetzt implementiert werden.</li>
   *     <li>InvalidCastException: wenn ein oder mehr Werte des bergeben Feldes einen Typ haben, 
   *       der nicht zum Typ der entsprechenden Column passt. Dazu muss zuerst ein System eingefhrt 
   *       werden, um Typsicherheit in PHP4 zu simulieren.</li>
   *     <li>ConstraintException: wenn das Hinzufgen der DataRow eine Constraint verletzt. Wird 
   *       zusammen mit der Klasse Constraint implementiert.</li>
   *     <li>NoNullAllowedException: wenn ein null-Wert in einer Spalte eingefgt wrde, die keine 
   *       null-Werte erlaubt. Dazu mus zuerst das Attribut AllowDBNull in Klasse {@link DataColumn}
   *       implementiert werden.</li>
   *   </ul>
   */
  function & LoadDataRow($values) {
    //wenn soviele Werte bergeben wurden wie diese DataTable Spalten hat:
    if (count($values) == count($this->columns())) {
      $updatingRow =& new DataRow($this, $values);
      $primaryKeyParts = array_keys($this->primaryKey());
      $loadingDone = false;
      
      //traversiere $this->Rows und suche nach einem passenden Primrschlssel
      $myRows =& $this->Rows;
      foreach ($myRows as $rowCnt => $updateableRow) {
        $isCandidate = true;
        foreach ($primaryKeyParts as $pkPartName => $pkPartVal) {
          if ($updateableRow->item($pkPartName) != $updatingRow->item($pkPartName)) {
            $isCandidate = false;
          }
        }
        if ($isCandidate) {
          // aktualisiere die gefundene Zeile; $updateableRow->itemArray($values) ist 
          // nicht mglich, weil foreach() intern auf einer Kopie des Arrays arbeitet.
          $myRows[$rowCnt]->itemArray($values);
          //weiteres Traversieren ist unntig, da die Primrschlsselwerte eindeutig 
          //sind und also maximal eine Zeile gefunden werden kann, die mit gegebenen
          //Primrschlsselwerten bereinstimmt.
          $loadingDone = true;
          return null;
        }
      }
      
      if (!$loadingDone) {
        //keine passende Zeile gefunden, lege eine neue an
        $this->Rows[] =& $updatingRow;
        return $updatingRow;
      }
      
    }
    // unzulssige Parameter bergeben, deshalb wurde nichts ausgefhrt:
    else {
      return null;
    }
  }
  
  /**
   * erzeugt eine neue {@link DataRow} entsprechend dem Schema dieser Tabelle jedoch ohne Werte
   *
   * Die erzeugte {@link DataRow} wird <i>nicht</i> automatisch zur {@link DataTable} hinzugefgt. 
   * Die {@link DataRow} ist unabhngig, weshalb auch ihr Attribut {@link DataRow::$Table Table}
   * noch den Wert null hat. Falls gewnscht, muss die DataRow anschlieend manuell zur 
   * Kollektion {@link Rows} einer {@link DataTable} hinzugefgt werden. Alternativ 
   * kann mit {@link LoadDataRow} eine {@link DataRow} in einem Schritt erzeugt und 
   * zur {@link DataTable} hinzugefgt werden.
   * 
   * Um die erzeugte {@link DataRow} mit Werten zu fllen verwende man {@link DataRow::item()} 
   * oder {@link DataRow::itemArray()}.
   *
   * @access public
   * @return &DataRow die neu erzeugte {@link DataRow}
   */
  function & NewRow() {
    return new DataRow($this, null);
  }
  
  /**
   * setzt diese {@link DataTable} in den Zustand nach Konstruktoraufruf zurck
   * 
   * Wie bei {@link Clear} werden alle Daten gelscht, darber hinaus jedoch auch alle 
   * nderungen am Schema ({@link DataColumn}-Objekte), den Beziehungen der Tabellen 
   * ({@link DataRelation}-Objekte) und den Constraints ({@link Constraint}-Objekte), 
   * die nach Ende des Konstruktoraufrufs gemacht wurden.
   * 
   * Diese Methode ist hauptschlich bei der Verwendung von typisierten {@link DataSet DataSets}
   * wertvoll. Dabei reprsetiert jede selbst abgeleitete Unterklasse von {@link DataTable} einen 
   * Typ einer Tabelle, mit spezifischem Schema, spezifischen Beziehungen und Constraints. Diese 
   * werden durch den Konstruktor erzeugt und durch eine berschriebene Version dieser Methode 
   * wiederhergestellt.
   *
   * @access public
   */
  function Reset() {
    $this->Clear(); // setzt $this->Rows = null
    $this->Columns = null;
    $this->PrimaryKey = null;
    $this->ParentRelations = null;
    $this->ChildRelations = null;
    $this->Constraints = null;
    $this->DataSet = null;
    $this->enforceConstraints(true);
    $this->loadingData(false);
  }
  
  /**
   * Wahrheitswert, der angibt, ob die Constraints dieser {@link DataTable} durchgesetzt werden
   *
   * get/set-Methode zu diesem Attribut: {@link EnforceConstraints}. Standardwert: <kbd>true</kbd>.
   * Die Constraints selbst sind zugreifbar ber das Attribut {@link Constraints}.
   *
   * @access private
   * @var bool
   */
  var $EnforceConstraints = true;
  
  /**
   * Angabe, ob die {@link Constraints} dieser DataTable durchgesetzt werden (lesen, schreiben)
   * 
   * get/set-Methode zum Attribut {@link EnforceConstraints}. Das Attribut gibt an, ob 
   * {@link Constraints} grundstzlich durchgesetzt werden. Es gilt jedoch nicht fr den Bereich 
   * zwischen {@link BeginLoadData} und {@link EndLoadData}; in diesem Bereich werden Constraints 
   * grundstzlich nicht durchgesetzt, auch wenn {@link EnforceConstraints}<kbd>==true</kbd>. Der 
   * Standardwert dieses Attributs ist <kbd>true</kbd>.
   *
   * Diese Methode inkl. dem Attribut {@link EnforceConstraints} wird erst in einer Version nach 1.0 
   * wirkungsvoll.
   *
   * Zugriff nur private und fr Klasse {@link DataSet}. Leider kann friend-Semantik nicht in 
   * PHP4-Syntax ausgedrckt werden.
   *
   * @access private
   * @param bool <kbd>true</kbd>, um die {@link Constraints} durchzusetzen, <kbd>false</kbd> um 
   *   sie unbercksichtigt zu lassen. Optionaler Parameter, nur bei Verwendung als set-Methode 
   *   bentigt.
   * @return bool aktuelle Angabe, ob die {@link Constraints} durchgesetzt werden (<kbd>true</kbd>) 
   *   oder unbercksichtigt bleiben (<kbd>false</kbd>).
   */
   function enforceConstraints($EnforceConstraints = null) {
     if (!is_null($EnforceConstraints)) {
       $this->EnforceConstraints = $EnforceConstraints;
     }
     return $this->EnforceConstraints;
   }
 
  /**
   * alle Constraints, die dieser Tabelle zugewiesen wurden
   *
   * get/set-Methode zu diesem Attribut: {@link constraints}. Standardwert: <kbd>null</kbd>.
   *
   * @access private
   */
  var $Constraints;
  
  /**
   * alle Constraints, die dieser Tabelle zugewiesen wurden (lesen, schreiben)
   *
   * get/set-Methode des Attributs {@link Constraints}. Schreiben geschieht durch 
   * ndern des Rckgabewertes, er ist eine Referenz auf das interne Feld von Constraint-Objekten.
   *
   * @access public
   * @return &Constraint[]
   */
  function & constraints() {
    return $this->Constraints;
  }
  
  /**
   * Angabe, ob gerade Daten geladen werden und deshalb auf unntige Reaktionen zu verzichten ist
   *
   * get/set-Methode dieses Attributs: {@link loadingData}. Standardwert: <kbd>false</kbd>.
   *
   * @access private
   * @var bool
   * @see BeginLoadData, EndLoadData
   */
  var $LoadingData = false;
  
  /**
   * Angabe, ob gerade Daten geladen werden und deshalb auf unntige Reaktionen zu verzichten 
   * ist (lesen, schreiben).
   *
   * get/set-Methode des Attributs {@link LoadingData}. Nur zur Verwendung innerhalb der 
   * Klasse {@link DataTable} gedacht. 
   *
   * @access private
   * @param bool Angabe, ob nun Daten in diese {@link DataTable} eingelesen werden sollen 
   *   (<kbd>true</kbd>) oder ob der Einlesevorgang beendet wurde (<kbd>false</kbd>). Optional, 
   *   nur bei Verwendung als set-Methode bentigt.
   * @return bool Angabe, ob gerade Daten in diese {@link DataTable} eingelesen werden 
   *   (<kbd>true</kbd>) oder nicht (<kbd>false</kbd>).
   */
  function loadingData($LoadingData = null) {
    if (!is_null($LoadingData)) {
      $this->LoadingData = $LoadingData;
    }
    return $this->LoadingData;
  }

  /**
   * bereitet die {@link DataTable} auf das Laden von Daten vor
   *
   * Diese Methode ist vor Aufrufen von {@link LoadDataRow} zu verwenden; ihr Effekt ist danach 
   * mit {@link EndLoadData} rckgngig zu machen. {@link BeginLoadData} schaltet diejenigen Teile des 
   * Objektverhaltens ab, die whrend des Ladevorgangs unntig sind. Konkret ist das bisher die 
   * Durchsetzung der {@link Constraints}. {@internal In spteren Versionen mag auch dazugehren: 
     Pflege des internen Zeilenindexes nach jedem Einfgevorgang; Auslsen von Ereignissen nach 
     jedem Einfgevorgang.} Dabei bleibt der Wert des Attributs {@link DataSet::$EnforceConstraints} 
   * unverndert, wird aber zeitweise fr diese {@link DataTable} unwirksam.
   *
   * @access public
   */
  function BeginLoadData() {
    $this->loadingData(true);
  }
  
  /**
   * bereitet das Laden von Daten in die {@link DataTable} nach
   * 
   * Der Effekt von {@link BeginLoadData} wird umgekehrt, mehr nicht. Konkret: Die Constraints 
   * dieser {@link DataTable} im Attribut {@link Constraints} werden nicht mehr grundstzlich 
   * vernachlssigt, sondern das Attribut {@link DataSet::$EnforceConstraints} ist fr 
   * diese {@link DataTable} wieder wirksam und bestimmt, ob Constraints durchgesetzt werden.
   *
   * @access public
   */
  function EndLoadData() {
    $this->loadingData(false);
  }
  
  /**
   * Kollektion aller {@link DataRelation}-Objekte, in der diese {@link DataTable} die 
   * Elternrolle hat.
   *
   * get/set-Methode zu diesem Attribut: {@link childRelations}. Standardwert: <kbd>null</kbd>.
   *
   * @access private
   * @var DataRelation[]
   */
  var $ChildRelations;  
 
  /**
   * Kollektion aller {@link DataRelation}-Objekte, in der diese {@link DataTable} die 
   * Elternrolle hat (lesen, schreiben).
   *
   * get/set-Methode des Attributs {@link ChildRelations}. Schreiben ist durch ndern des 
   * Rckgabewertes mglich. Beachte, dass jedes {@link DataRelation}-Objekt genau 
   * zwei {@link DataTable DataTables} verbindet und deshalb zustzlich zum Eintragen 
   * in {@link ChildRelations} zu {@link ParentRelations} genau einer 
   * anderen {@link DataTable} hinzugefgt werden muss.
   *
   * @access public
   * @return &DataRelation[] Kollektion aller {@link DataRelation}-Objekte, in der 
   *   diese {@link DataTable} die Elternrolle hat.
   */
  function & childRelations() {
    return $this->ChildRelations;
  }
  
  /**
   * Kollektion aller {@link DataRelation}-Objekte, in der diese {@link DataTable} die 
   * Kindrolle hat.
   *
   * get/set-Methode zu diesem Attribut: {@link parentRelations}
   *
   * @access private
   * @var DataRelation[]
   */
  var $ParentRelations;
     
  /**
   * Kollektion aller DataRelation-Objekte, in der diese DataTable die Kindrolle hat (lesen, 
   * schreiben).
   *
   * get/set-Methode des Attributs {@link ParentRelations}. Schreiben ist durch ndern des 
   * Rckgabewertes mglich. Standardwert: <kbd>null</kbd>. Beachte, dass 
   * jedes {@link DataRelation}-Objekt genau zwei {@link DataTable DataTables} verbindet und 
   * deshalb zustzlich zum Eintragen in {@link ParentRelations} zu {@link ChildRelations} 
   * genau einer anderen {@link DataTable} hinzugefgt werden muss.
   *
   * @access public
   * @return &DataRelation[] Kollektion aller {@link DataRelation}-Objekte, in der 
   *   diese {@link DataTable} die Kindrolle hat.
   */
  function & parentRelations() {
    return $this->ParentRelations;
  }   
}
?>
