<?php

/**
 * Die Klasse {@link DataTableTest}
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */
 
/**
 * PhpUnit, das hier benutzte Testing-Framework
 *
 * Diese Klasse erwartet das vollstndige Testing-Framework PhpUnit 0.5 im 
 * Verzeichnis ./phpunit . Dies ist auch das Standard-Verzeichnis, in das 
 * sich PhpUnit 0.5 entpackt.
 *
 * PhpUnit 0.5 kann bezogen werden ber {@link http://sourceforge.net/projects/phpunit}. 
 * Hilfreich zum Umgang mit diesem Testing-Framework ist auerdem 
 * das {@link http://sourceforge.net/projects/phpunitcookbook/ PHPUnit Cookbook}
 */
require_once("phpunit/phpunit.php");

require_once("DataTable.php");
require_once("DataColumn.php");
require_once("DataRow.php");
require_once("DataRelation.php");

/**
 * Test der Klasse {@link DataTable}
 *
 * Diese Klasse ist gedacht zur Verwendung mit dem 
 * Test-Framework {@link http://phpunit.sourceforge.net PhpUnit}.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */
class DataTableTest extends TestCase {

  /**
   * Eine {@link DataTable}, vorbereitet fr nahezu beliebige Tests
   */
  var $myDataTable;
  
  /**
   * Originalzustand von {@link myDataTable} unmittelbar nach Konstruktoraufruf
   */
  var $myOriginalDataTable;

  /**
   * Konstruktor der Klasse {@link DataTableTest}
   */
  function DataTableTest ($name = "DataTableTest") {
    $this->TestCase($name);
  }
  
  /**
   * Erzeuge Objekte, auf denen alle Tests operieren sollen.
   *
   * berschriebene Methode der Klasse {@link TestCase}, die zum 
   * Test-Framework {@link http://phpunit.sourceforge.net PhpUnit} gehrt. Es werden die 
   * Attribute des aktuellen Objektes so gesetzt, dass sie vorbereitet fr die Ausfhrung des 
   * nchsten Tests sind. Ein Test besteht in der Ausfhrung einer Methode, die mit "test"
   * beginnt.
   */
  function setUp() {
    // setUp() muss jeder Testmethode eine identische Umgebung bereitstellen. Dazu gehrt auch, 
    // dass statische Variablen denselben Wert haben:
    DataTable::presetObjCounter(0);
  
    $this->myDataTable =& new DataTable("Altersangaben");
    
    // DataTable::Reset() soll eine DataTable in ihren Originalzustand (d.i. ihr Zustand nach 
    // Konstruktoraufruf) zurckversetzen. Erhalte also den Originalzustand zum Vergleichen:
    $this->myOriginalDataTable = $this->myDataTable;
    
    $myDataTableSchema =& $this->myDataTable->columns();
    $myDataTableSchema[] = new DataColumn(& $this->myDataTable, "Name");
    $myDataTableSchema[] = new DataColumn(& $this->myDataTable, "Vorname");
    $myDataTableSchema[] = new DataColumn(& $this->myDataTable, "Alter");
    $this->myDataTable->primaryKey(
      array(& $myDataTableSchema[0], & $myDataTableSchema[1])
    );
    $myDataTableRows =& $this->myDataTable->rows();
    $myDataTableRows[] = new DataRow(& $this->myDataTable, array("Seeler", "Holger", 20));
    $myDataTableRows[] = new DataRow(& $this->myDataTable, array("Seele",  "Holge",  21));
    $myDataTableRows[] = new DataRow(& $this->myDataTable, array("Seel",   "Holg",   20));
    $myDataTableRows[] = new DataRow(& $this->myDataTable, array("See",    "Hol",    21));
    $myDataTableRows[] = new DataRow(& $this->myDataTable, array("Se",     "Ho",     22));
    
    $nullDataColumnArray = null; //um "nichts" als Referenzparamater bergeben zu knnen
    
    $myDataTableParentRelations =& $this->myDataTable->parentRelations();
    $myDataTableParentRelations[] =
      new DataRelation($nullDataColumnArray, $nullDataColumnArray, "ParentRelation 1");
    $myDataTableParentRelations[] =
      new DataRelation($nullDataColumnArray, $nullDataColumnArray, "ParentRelation 2");
    
    $myDataTableChildRelations =& $this->myDataTable->childRelations();
    $myDataTableChildRelations[] =
      new DataRelation($nullDataColumnArray, $nullDataColumnArray, "ChildRelation 1");
    $myDataTableChildRelations[] =
      new DataRelation($nullDataColumnArray, $nullDataColumnArray, "ChildRelation 2");
  }
  
  /**
   * Ressourcenfreigabe permanent allozierter Ressourcen nach Durchfhrung des letzten Test.
   *
   * berschriebene Methode der Klasse {@link TestCase}, die zum 
   * Test-Framework {@link http://phpunit.sourceforge.net phpUnit} gehrt. Es werden die 
   * Attribute des aktuellen Objektes so gesetzt, dass alle fr die Durchfhrung eines Tests 
   * belegten Ressourcen wieder freigegeben sind. Ein Test besteht in der Ausfhrung einer 
   * Methode, die mit "test" beginnt.
   */
  function teardown() {
  }
  
  /**
   * Testet ob das Attribut {@link DataTable::$TableName} korrekt verwaltet wird.
   */
  function test_TableName() {
    
    $TblAutoid = new DataTable();
    $TblMyname = new DataTable("Myname");

    // Erwarte Name "DataTable1" statt "DataTable0", weil bereits im automatisch vor dieser Methode 
    // aufgerufenen $this->setUp() eine andere DataTable erzeugt wurde.
    $this->assertEquals(
      "DataTable1",
       $TblAutoid->tableName(),
      "automatisch generierte Tabellennamen werden nicht korrekt generiert, gespeichert " .
      "oder ausgelesen"
    );

    $this->assertEquals(
      "Myname",
      $TblMyname->tableName(),
      "benutzerdefinierte Tabellennamen werden nicht korrekt abgespeichert oder ausgelesen"
    );
  }

  /**
   * Testet das Einladen von Daten unter Verwendung von {@link DataTable::LoadDataRow()}
   *
   * Relevante Methoden: {@link DataTable::BeginLoadData()}, {@link DataTable::EndLoadData()}
   * und {@link DataTable::LoadDataRow()}.
   */
  function test_LoadDataRow() {
    $this->myDataTable->BeginLoadData();
    $this->assertEquals(
      true,
      $this->myDataTable->loadingData(),
      "DataTable::BeginLoadData() setzt DataTable::LoadingData nicht auf true"
    );
    
    // LoadDataRow() mit existierenden Primrschlsselwerten: Update erwartet
    $this->myDataTable->LoadDataRow(array("Seeler", "Holger", 25));
    $this->assertEquals(
      5,
      count($this->myDataTable->rows()),
      "Update von Zeilen mit gleichem Primrschlssel funktioniert nicht: falsche Zeilenzahl"
    );
    $myDataTableRows = $this->myDataTable->rows();
    $this->assertEquals(
      array("Seeler", "Holger", 25),
      $myDataTableRows[0]->itemArray(),
      "Update von Zeilen mit gleichem Primrschlssel funktioniert nicht: falsche Zeilenwerte"
    );
    
    // LoadDataRow() mit neuen Primrschlsselwerten: neue Zeile erwartet
    $this->myDataTable->LoadDataRow(array("Seelerli", "Holgerli", 25));
    $this->assertEquals(
      6,
      count($this->myDataTable->rows()),
      "Anhngen von Zeilen mit neuem Primrschlssel funktioniert nicht: falsche Zeilenzahl"
    );
    
    $this->myDataTable->EndLoadData();
    $this->assertEquals(
      false,
      $this->myDataTable->loadingData(),
      "DataTable::EndLoadData() setzt DataTable::LoadingData nicht auf false"
    );
  }
  
  /**
   * Testet das Einladen von Daten unter Verwendung von {@link DataTable::ImportRow()}
   *
   * Nicht getestet wird, ob die erzeugten {@link DataRow}-Objekte ihre Werte behalten 
   * und korrekt verwalten; das ist Aufgabe von {@link DataRowTest}.
   */
  function test_ImportRow() {
  
    // LoadDataRow() mit gleichen Primrschlsselwerten: neue Zeile erwartet
    $this->myDataTable->ImportRow(
      new DataRow(& $this->myDataTable, array("Seeler", "Holger", 25))
    );
    $this->assertEquals(
      6,
      count($this->myDataTable->rows()),
      "ImportRow fhrt bei gleichen Primrschlsselwerten zu keiner neuen Zeile"
    );
    
    // LoadDataRow() mit neuen Primrschlsselwerten: neue Zeile erwartet
    $this->myDataTable->ImportRow(
      new DataRow(& $this->myDataTable, array("Seelerli", "Holgerli", 25))
    );
    $this->assertEquals(
      7,
      count($this->myDataTable->rows()),
      "ImportRow fhrt bei neuen Primrschlsselwerten zu keiner neuen Zeile"
    );
  }
  
  /**
   * Testet die Methode {@link DataTable::NewRow()}
   */
  function test_NewRow() {
    $myNewRow = $this->myDataTable->NewRow();
    
    // Im Gegenteil zur Behauptung im PHPUnit Cookboook erzeugt Assert::assert() einen 
    // Fehler wenn das erste Argument false ist, nicht wenn es true ist. Das erste Argument 
    // stellt also keine Fehlerbedingung dar, sondern eine zuzusichernde Bedingung, ganz 
    // entsprechend der Wortbedeutung von assert.
    $this->assert(!is_null($myNewRow), "es wurde keine neue DataRow erzeugt");
    $this->assert(is_a($myNewRow, "DataRow"), "das erzeugte Objekt ist keine DataRow");
    $this->assertEquals(
      false,
      $myNewRow->rowState(),
      "die erzeugte DataRow ist nicht unabhngig"
    );
  }

  /**
   * Testet die Methode {@link DataTable::Clear()}
   */
  function test_Clear() {
    $this->myDataTable->Clear();
    $this->assert(
      is_null($this->myDataTable->rows()),
      "die Kollektion der Zeilen dieser Tabelle ist nicht leer"
    );
  }
  
  /**
   * Testet die Methode {@link DataTable::Reset()}
   */
  function test_Reset() {
    $this->myDataTable->Reset();
    
    $this->assertEquals(
      $this->myOriginalDataTable,
      $this->myDataTable,
      "die Tabelle wurde nicht in ihren Zustand nach Konstruktoraufruf zurckgesetzt"
    );
  }
  
  /**
   * Testet die Verwaltung von Beziehungen zwischen {@link DataTable DataTables}
   *
   * Die relevanten Attribute sind {@link DataTable::ChildRelations} und 
   * {@link DataTable::ParentRelations}. Dieser Test stellt nur fest, ob eine 
   * {@link DataTable} sich ihre {@link DataRelation}-Objekte korrekt merkt. Ob sie 
   * auch korrekt funktionieren muss in {@link DataRelation} getestet werden.
   */
  function test_Relations() {
    $nullDataColumnArray = null; //um "nichts" als Referenzparameter bergeben zu knnen
  
    // hat DataTable::ParentRelations die nach setUp() erwarteten Werte?
    $myDataTableParentRelations =& $this->myDataTable->parentRelations();
    $this->assert(
      count($myDataTableParentRelations) == 2,
      "falsche Anzahl von Elementen in DataTable::ParentRelations nach Konstruktoraufruf"
    );
    $this->assert(
      $myDataTableParentRelations[0]->RelationName() == "ParentRelation 1" 
        && $myDataTableParentRelations[1]->RelationName() == "ParentRelation 2",
      "falsche Werte in Element(en) in DataTable::ParentRelations nach Konstruktoraufruf"
    );
    
    // Hinzufgen von Relationen zu DataTable::ParentRelations mglich?
    $myDataTableParentRelations[] =
      new DataRelation($nullDataColumnArray, $nullDataColumnArray, "ParentRelation 3");
    $this->assert(
      count($myDataTableParentRelations) == 3 &&
        $myDataTableParentRelations[2]->RelationName() == "ParentRelation 3",
      "Hinzufgen von Relationen zu DataTable::ParentRelations fehlerhaft"
    );
    
    // hat DataTable::ChildRelations die nach setUp() erwarteten Werte?
    $myDataTableChildRelations =& $this->myDataTable->childRelations();
    $this->assert(
      count($myDataTableChildRelations) == 2,
      "falsche Anzahl von Elementen in DataTable::ChildRelations nach Konstruktoraufruf"
    );
    $this->assert(
      $myDataTableChildRelations[0]->RelationName() == "ChildRelation 1" 
        && $myDataTableChildRelations[1]->RelationName() == "ChildRelation 2",
      "falsche Werte in Element(en) in DataTable::ChildRelations nach Konstruktoraufruf"
    );
    
    // Hinzufgen von Relationen zu DataTable::ChildRelations mglich?
    $myDataTableChildRelations[] =
      new DataRelation($nullDataColumnArray, $nullDataColumnArray, "ChildRelation 3");
    $this->assert(
      count($myDataTableChildRelations) == 3 &&
        $myDataTableChildRelations[2]->RelationName() == "ChildRelation 3",
      "Hinzufgen von Relationen zu DataTable::ChildRelations fehlerhaft"
    );
  }
  
  /**
   * Testet ob das Schema der {@link DataTable} korrekt verwaltet wird
   *
   * Relevantes Attribut ist {@link DataTable::$Columns}.
   */
  function test_Columns() {
    
    // hat DataTable::Columns die nach setUp() erwarteten Werte? Damit ist dann gleichzeitig 
    // garantiert dass Columns zur DataTable hinzugefgt werden knnen, denn so bekam setUp() 
    // sie in die DataTable hinein.
    $myDataTableSchema =& $this->myDataTable->columns();
    $this->assertEquals(
      3,
      count($myDataTableSchema),
      "DataTable::Columns enthlt eine falsche Zahl an DataColumns"
    );
    
    $myDataColumn_origNames = array("Name", "Vorname", "Alter");
    foreach($myDataTableSchema as $i => $myDataColumn) {
        $this->assertEquals(
        $myDataColumn_origNames[$i],
        $myDataColumn->ColumnName(),
        "DataTable::Columns[$i] enthlt einen falschen Wert"
      );
    }
  }
  
  /**
   * Testet das Attribut {@link DataTable::$PrimaryKey}
   */
  function test_PrimaryKey() {
    // hat DataTable::PrimaryKey den nach setUp() erwarteten Wert? Dann ist gleichzeitig 
    // garantiert, dass dieser Wert gendert werden kann, denn das wurde in setUp() gemacht.
    $myDataTablePrimaryKey = $this->myDataTable->primaryKey();
    $myDataTableSchema = $this->myDataTable->columns();
    
    $this->assert(
      $myDataTablePrimaryKey[0]->Equals($myDataTableSchema[0])
        && $myDataTablePrimaryKey[1]->Equals($myDataTableSchema[1]),
      "DataTable::PrimaryKey konnte nicht korrekt gesetzt werden"
    );
  }

}
     
?>
