<?php

/**
 * Klassen zur Event-Behandlung und einzelne Event-Klassen fr {@link DataSet} und 
 * assoziierte Klassen.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */
 
/**
 * Basisklasse fr alle konkreten Events
 *
 * Das hier eingesetzte vereinfachte Events- und Exceptions-Konzept ist dokumentiert in 
 * Klasse {@link DataException}.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 * @see DataException DataExceptions.php
 */
class Event {
}

/**
 * Tritt in {@link DataSet::Merge()} auf, wenn eine Quell- und Ziel-{@link DataRow} gleiche 
 * Primrschlsselwerte haben und {@link DataSet::$EnforceConstraints} <kbd>true</kbd> ist.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 * @see Event
 */

class MergeFailed extends Event {

  /**
   * Konstruktor der Klasse {@link MergeFailed}
   *
   * @param DataTable $DataTable Diejenige {@link DataTable} im Ziel-{@link DataSet}, in der 
   *   der Konflikt auftrat, der zu diesem {@link MergeFailed}-Event fhrte. Die hiermit 
   *   angegebene {@link DataTable} kann aus dem erstellten Objekt durch {@link dataTable} 
   *   ausgelesen werden.
   * @param string $Conflict textuelle Beschreibung dieses Konflikts. Sie kann aus dem erstellten 
   *   Objekt durch {@link conflict} ausgelesen werden.
   */
  function MergeFailed(& $DataTable, $Conflict) {
    $this->DataTable = $DataTable;
    $this->Conflict = $Conflict;
  }

  /**
   * Die {@link DataTable} des Ziel-{@link DataSet DataSets}, in der der Konflikt {@link 
   * Conflict} auftrat, der dieses {@link MergeFailed}-Event auslste.
   *
   * get-Methode dieses Attributs: {@link dataTable}.
   *
   * @access private
   * @var DataTable
   */
  var $DataTable;
  
  /**
   * Die {@link DataTable} des Ziel-{@link DataSet DataSets}, in der der Konflikt auftrat, 
   * der dieses Event auslste (lesen).
   *
   * @access public
   * @return DataTable& {@link DataTable} in der der Konflikt {@link Conflict} auftrat, der 
   *   dieses Event auslste.
   */
  function & dataTable() {
    return $this->$DataTable;
  }
  
  /**
   * Eine textuelle Beschreibung des Konfliktes, der dieses Event auslste.
   *
   * get-Methode dieses Attributs: {@link conflict}.
   *
   * @access private
   * @var string
   */
  var $Conflict;

  /**
   * Eine textuelle Beschreibung des Konfliktes, der dieses {@link MergeFailed}-Event 
   * auslste (lesen).
   * 
   * get-Methode des Attributs {@link Conflict}.
   *
   * @access public
   * @return string textuelle Beschreibung des Konflikts, der dieses Event auslste.
   */
  function conflict() {
    return $Conflict;
  }
}

?>
