<?php

/**
 * Die Klasse {@link MissingSchemaAction}
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */

/**
 * Definiert einige Werte, die Verhaltensalternativen bei notwendigen Schemanderungen 
 * an einem {@link DataSet} reprsentieren.
 *
 * @package phpDataSet
 * @author Matthias Ansorg <matthias at ansorgs dot de>
 */
class MissingSchemaAction {

  /**
   * Die zustzlichen {@link DataColumns} werden zum Schema der jeweiligen {@link DataTable} 
   * hinzugefgt und so das Schema komplettiert.
   *
   * @access public
   * @var string
   */
  function Add() {
    return "Add";
  }
  
  /**
   * Die zustzlichen {@link DataColumns} werden zum Schema der jeweiligen {@link DataTable} 
   * hinzugefgt, auerdem wird der Primary Key der jeweiligen {@link DataTable} so eingestellt 
   * wie in der entsprechenden {@link DataTable} der Quelldaten.
   *
   * Diskrepanzen im Schema zweier {@link DataTable DataTables} knnen in der Menge 
   * der {@link DataColumn}-Objekte bestehen (wobei jede {@link DataColumn} durch ein Tupel 
   * (Name,Datentyp) reprsentiert ist). Zustzlich knnen Diskrepanzen darin bestehen, 
   * welche Untermenge der {@link DataColumn DataColumns} als Primrschlssel festgelegt 
   * wurde. Wird die Primrschlsseleinstellung nicht mit bernommen (etwa bei Verwendung 
   * von {@link Add MissingSchemaAction::Add()}), so kann sich daraus u.U. ein fehlerhaftes 
   * Verhalten ergeben. Spter hinzuzufgende Zeilen werden angehngt statt dass bisherige 
   * Zeilen aktualisiert werden, wie es bei Verwendung des korrekten, neuen Primrschlssels 
   * geschhe.
   *
   * Beispiel: die Quell-{@link DataTable} enthlt eine Spalte "ID" mehr und eine genderte 
   * Primrschlsseleinstellung derart, dass nun diese eine Spalte den Primrschlssel bildet. 
   * In der Ziel-{@link DataTable} wird ein zusammengesetzter Primrschlssel verwendet. Die neue 
   * Primrschlsseleinstellung werde nicht bernommen, anschlieend werde eine Zeile eingefgt, 
   * die nach dem neuen Primrschlssel auf eine existierende Zeile passt (also zu einem Update 
   * fhren msste), nach der bisherigen Primrschlsseleinstellung aber auf keine existierende 
   * Zeile passt und deshalb angehngt wird.
   *
   * Die beabsichtigte Aktualisierung ist nur beim Einfgen von {@link DataRow DataRows} mglich.
   * Bei der Zusammenfhrung eines DataSet mit anderen Daten durch {@link DataSet::Merge()} ist 
   * keine Aktualisierung mglich; Zeilen mit gleichem Primrschlssel in Quell- und Zieldaten 
   * werden dort als Fehler gewertet.
   *
   * @access public
   * @var string
   * @see DataSet::Merge()
   * @todo Prfe das in der Dokumentation zu {@link MissingSchemaAction::AddWithKey()} wiedergegebene 
   *   Verstndnis von "AddWithKey" nach. Die .NET-Dokumentation ist hier schlecht verstndlich 
   *   (siehe {@link http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpref/html/frlrfsystemdatamissingschemaactionclasstopic.asp
       System.Data.MissingSchemaAction Enumeration (MSDN)}).
   */
  function AddWithKey() {
    return "AddWithKey";
  }
  
  /**
   * Zustzliche {@link DataColumns}, d.h. inkompatible Schemata, werden als Fehler gewertet
   *
   * @access public
   * @var string
   * @todo Beschreibe, wie die Fehlerbehandlung aussieht.
   */
  function Error() {
    return "Error";
  }
  
  /**
   * Die zustzlichen {@link DataColumns} werden ignoriert
   *
   * @access public
   * @var string
   */
  function Ignore() {
    return "Ignore";
  }
}
?>
